@@ -14,7 +14,7 @@ from reportlab.platypus.flowables import Flowable, DocAssert
 from reportlab.lib.colors import Color
 from reportlab.lib.units import cm
 from reportlab.lib.enums import TA_LEFT, TA_RIGHT, TA_CENTER, TA_JUSTIFY
-from reportlab.lib.utils import _className
+from reportlab.lib.utils import _className, asBytes, asUnicode
 from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
 from reportlab.platypus.xpreformatted import XPreformatted
 from reportlab.platypus.frames import Frame, ShowBoundaryValue
@@ -701,6 +701,42 @@ class FragmentTestCase(unittest.TestCase):
             y = _t(S[1],x,y,naW,aH=naH)
         canv.save()
 
+    @unittest.skipUnless(pyphen,'s')
+    def test7(self):
+        """test various ways to adjust the hypenationMinWordLength"""
+        registerFont(TTFont("Vera", "Vera.ttf"))
+        aW = 51.0236220472
+        text  = u'\u0440\u044b\u0431\u0430 \u043f\u0438\u043b\u0430 \u0438 \u0431\u0430\u0431\u0443 \u0434\u0430\u0436\u0435 \u0432\u0435\u043b\u0430 \u043d\u0430 \u043c\u0430\u044f\u043a'
+        text1 = u'\u0440\u044b\u0431\u0430 <span color="blue">\u043f\u0438\u043b\u0430</span> \u0438 \u0431\u0430\u0431\u0443 \u0434\u0430\u0436\u0435 \u0432\u0435\u043b\u0430 \u043d\u0430 \u043c\u0430\u044f\u043a'
+        tmpls = [
+                u'%(text)s',
+                u'<para hyphenationMinWordLength="%(hymwl)d">%(text)s</para>',
+                u'<para>%(text)s</para>',
+                u'%(text1)s',
+                u'<para hyphenationMinWordLength="%(hymwl)d">%(text1)s</para>',
+                u'<para>%(text1)s</para>',
+                ]
+        for ex,x,hymwl in [
+                (72,0,None),    #default is 5
+                (72,0,5),
+                (60,0,4),
+                (72,2,None),    #default is 5
+                (72,1,5),
+                (60,1,4),
+                (72,3,None),    #default is 5
+                (72,4,5),
+                (60,5,4),
+                ]:
+            kwds = dict(hyphenationMinWordLength=hymwl) if hymwl!=None else {}
+            template = tmpls[x]
+            t = template % locals()
+            p = Paragraph(
+                    template % locals(),
+                    style = ParagraphStyle('P10H5', fontSize=10, fontName="Vera", hyphenationLang="ru_RU",**kwds),
+                    )
+            w,h = p.wrap(aW,0x7fffffff)
+            self.assertEqual(h,ex,'Russion hyphenation test failed for ex=%(ex)s template=%(template)r hymwl=%(hymwl)r h=%(h)s\ntext=%(t)r' % locals())
+
 class ULTestCase(unittest.TestCase):
     "Test underlining and overstriking of paragraphs."
     def testUl(self):
@@ -926,6 +962,44 @@ phonemic and <u>morphological</u> <strike>analysis</strike>.'''
         doc = MyDocTemplate(outputfile('test_platypus_paragraphs_autoleading.pdf'))
         doc.build(story)
 
+DEJAVUSANS = ('DejaVuSans','DejaVuSans-Bold','DejaVuSans-Oblique','DejaVuSans-BoldOblique')
+def haveDejaVu():
+    from reportlab.pdfbase.ttfonts import TTFont
+    for x in DEJAVUSANS:
+        try:
+            TTFont(x,x+'.ttf')
+        except:
+            return False
+    return True
+
+DEJAVUSANS = ('DejaVuSans','DejaVuSans-Bold','DejaVuSans-Oblique','DejaVuSans-BoldOblique')
+def alphaSortedItems(d):
+    return (i[1] for i in sorted((j[0].lower(),j) for j in d.items()))
+
+def tentities(title, b, fn):
+    from reportlab.platypus.paraparser import greeks
+    from reportlab.platypus.doctemplate import SimpleDocTemplate
+    from reportlab.pdfbase.pdfmetrics import stringWidth, registerFont, registerFontFamily
+    from reportlab.pdfbase.ttfonts import TTFont
+    from reportlab.platypus.tables import TableStyle, Table
+    from reportlab.platypus.paragraph import Paragraph
+    from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
+    from reportlab import ascii
+
+    for v in DEJAVUSANS:
+        registerFont(TTFont(v,v+'.ttf'))
+    registerFontFamily(*(DEJAVUSANS[:1]+DEJAVUSANS))
+
+    def bu(s):
+        return asUnicode(s) if not b else asBytes(s)
+
+    bt = getSampleStyleSheet()['BodyText']
+    bt.fontName = 'DejaVuSans'
+    doc = SimpleDocTemplate(fn)
+    story = [Paragraph('<b>%s</b>' % title,bt)]
+    story.extend([Paragraph(bu('&amp;%s; = <span color="red">&%s;</span>' % (k,k)), bt) for k, v in alphaSortedItems(greeks)])
+    doc.build(story)
+
 class JustifyTestCase(unittest.TestCase):
     "Test justification of paragraphs."
     def testUl(self):
@@ -1162,15 +1236,39 @@ impose an interpretation on the system of base rules exclusive of the
                         ])
         styleSheet = getSampleStyleSheet()
         normal = ParagraphStyle(name='normal',fontName='Helvetica',fontSize=10,leading=12,parent=styleSheet['Normal'])
-        bold = ParagraphStyle(name='bold',fontName='Helvetica-Bold',fontSize=12,leading=14.4,parent=normal)
+        bold = ParagraphStyle(name='normal',fontName='Helvetica-Bold',fontSize=10,leading=12,parent=styleSheet['Normal'])
+        registerFontFamily('Helvetica','Helvetica','Helvetica-Bold','Helvetica-Oblique','Helvetica-BoldOblique')
         story =[]
         a = story.append
         a(Paragraph('Paragraph Hard Space Handling', bold))
         a(Paragraph(''' <span backcolor="pink">ABCDEFGHI</span> ''', normal))
         a(Paragraph(''' <span backcolor="palegreen">&nbsp;ABCDEFGHI&nbsp;</span> ''', normal))
+        a(Paragraph('''<span backcolor="lightblue">&nbsp;</span>''', normal))
+        a(Paragraph('''<span backcolor="lightblue">&nbsp;</span><span backcolor="palegreen">&nbsp;</span> ''', normal))
+        a(Paragraph('''<span backcolor="pink"><b>A</b></span><span backcolor="lightblue"> </span><span backcolor="pink"><b>B</b></span>''', normal))
+        a(Paragraph('''<span backcolor="pink"><b>A</b></span> <span backcolor="lightblue"> </span><span backcolor="pink"><b>B</b></span>''', normal))
+        a(Paragraph('''<span backcolor="pink"><b>A</b></span><span backcolor="lightblue">&nbsp;</span><span backcolor="pink"><b>B</b></span>''', normal))
+        a(Paragraph('''<span backcolor="pink"><b>A</b></span> <span backcolor="lightblue">&nbsp;</span><span backcolor="pink"><b>B</b></span>''', normal))
         doc = MyDocTemplate(outputfile('test_platypus_paragraphs_nbsp.pdf'))
         doc.build(story)
 
+    @unittest.skipUnless(haveDejaVu(),'s')
+    def testParaEntities(self):
+        tentities(b'unicode formatted paragraphs',False,outputfile('test_platypus_unicode_paragraph_entities.pdf'))
+        tentities(b'byte formatted paragraphs',True,outputfile('test_platypus_bytes_paragraph_entities.pdf'))
+
+    def testXPreUnderlining(self):
+        styleSheet = getSampleStyleSheet()
+        bt = styleSheet['BodyText']
+        story = [
+                XPreformatted("xpre<u><font size='1' color='red'>SS</font>        </u>",bt),
+                XPreformatted("xpre<u><font size='1' color='red'>SS</font>        </u>|",bt),
+                XPreformatted("<u><font size='1' color='red'>SS</font>        </u>",bt),
+                XPreformatted("<u><font size='1' color='red'>SS</font>        </u>|",bt),
+                ]
+        doc = MyDocTemplate(outputfile('test_platypus_XPreUnderlining.pdf'))
+        doc.build(story)
+
 #noruntests
 def makeSuite():
     return makeSuiteForClasses(ParagraphCorners,SplitFrameParagraphTest,FragmentTestCase, ParagraphSplitTestCase, ULTestCase, JustifyTestCase,
